/*
 * "ESWIN Computing" Memory Copy SDK
 *
 * Copyright 2024, Beijing ESWIN Computing Co., Ltd.. All rights reserved.
 *
 * Unauthorized copying, modification, or distribution of this program,
 * in whole or in part, is strictly prohibited without prior written
 * permission from Beijing ESWIN Computing Technology Co., Ltd.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/sysinfo.h>
#include <getopt.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <pthread.h>
#include <sys/time.h>
#include <stdatomic.h>

#include "es_sys_memory.h"
#include "es_vb_memory.h"
#include "es_memcp.h"

#define ES_MEMCP_INFO_ON 1
#ifdef ES_MEMCP_INFO_ON
#define ES_MEMCP_ERR(format, ...) \
    do { \
        printf("%s %s:%5d " format, "[E][MEM_TEST] ", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        fflush(stdout); \
    } while (0)

#define ES_MEMCP_INFO(format, ...) \
    do { \
        printf("%s %s:%5d " format, "[I][MEM_TEST] ", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        fflush(stdout); \
    } while (0)

#endif
#ifndef ES_MEMCP_ERR
#define ES_MEMCP_ERR(format, ...)
#endif
#ifndef ES_MEMCP_INFO
#define ES_MEMCP_INFO(format, ...)
#endif

#define PRIME_SEED_A    0x7091 // 29917
#define PRIME_SEED_B    0x75B5 // 30133

#define NID0_MMZ0_NAME    "mmz_nid_0_part_0"
#define DEFAULT_NID0_MMZ_NAME    NID0_MMZ0_NAME

#define LEN_2M 2*1024*1024
#define LEN_4M 4*1024*1024

int32_t ES_VB_MemDebugInfo(ES_VOID);
ES_U64 sys_test_len[] = {LEN_2M,LEN_4M};

struct test_case_t {
    const ES_CHAR *name;
    ES_S32 (*prepare)(void);
    ES_S32 (*test_fn)(void);
    ES_S32 (*cleanup)(void);
    ES_S32 testResult;
};

struct case_vb_config_param_t {
    ES_S32 poolCnt;
    ES_S32 blkSize;
    ES_S32 blkCnt;
    SYS_CACHE_MODE_E enRemapMode;
    ES_CHAR mmzName[ES_MAX_MMZ_NAME_LEN];
};

struct cache_info {
    const ES_CHAR *zone;
    ES_S32 kernel_cache_type;
    ES_S32 mmap_cache_type;
};

struct sys_testdata_info {
    struct cache_info src;
    struct cache_info dst;
};

struct sys_testdata_info sys_test_data[] = {
    /*src type:        vmap                mmap */       /*dst type:        vmap             mmap*/
    {{"system",           SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}, {"system",         SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}}, // 0
    {{"linux,cma",           SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}, {"linux,cma",         SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}}, // 1
    {{DEFAULT_NID0_MMZ_NAME,SYS_CACHE_MODE_CACHED,    SYS_CACHE_MODE_NOCACHE}, {DEFAULT_NID0_MMZ_NAME,SYS_CACHE_MODE_NOCACHE, SYS_CACHE_MODE_CACHED}}, // 2
};

struct case_vb_config_param_t vb_normal_cases[] = {
    {
        .poolCnt = 1,
        .blkSize = 4096*4,
        .blkCnt = 2,
        .enRemapMode = SYS_CACHE_MODE_NOCACHE,
        .mmzName = "mmz_nid_0_part_0",
    }
};

static uint32_t lcg_parkmiller(uint32_t *state)
{
    uint64_t product = (uint64_t)*state * 48271;
    uint32_t x = (product & 0x7fffffff) + (product >> 31);

    x = (x & 0x7fffffff) + (x >> 31);
    return *state = x;
}

static void buf_fillin_random(void *buf, uint32_t size, uint32_t prime_seed)
{
    uint32_t *tmpbuf = (uint32_t *)buf;
    uint32_t size_remaining = size;
    uint32_t seed = prime_seed;
    uint32_t i = 0;

    memset(buf, 0, size);
    while (size_remaining > 0) {
        tmpbuf[i] = lcg_parkmiller(&seed);
        size_remaining -= 4;
        i++;
    }
}

static int buf_compwith_random(void *buf, uint32_t size, uint32_t prime_seed)
{
    uint32_t *tmpbuf = (uint32_t *)buf;
    uint32_t size_remaining = size;
    uint32_t random, seed = prime_seed;
    uint32_t i = 0;

    while (size_remaining > 0) {
        random = lcg_parkmiller(&seed);
        if (tmpbuf[i] != random)    {
            printf("Err Comp, (uint32_t)buf[%d]:0x%x is NOT equal to expected val:0x%x\n", i, tmpbuf[i], random);
            return -1;
        }
        size_remaining -= 4;
        i++;
    }

    return 0;
}

ES_VOID ES_VB_PrintConfig(const VB_CONFIG_S *pVbConfig)
{
    ES_S32 i;
    const VB_POOL_CONFIG_S *pPoolcfg;

    printf("%s %d, poolCnt %d\n", __func__,__LINE__, pVbConfig->poolCnt);
    for (i = 0; i < pVbConfig->poolCnt; i++) {
        pPoolcfg = &pVbConfig->poolCfgs[i];
        printf("\tpool %d\n", i);
        printf("\t\tmmzName %s blkSize 0x%llx, blkCnt %d, enRemapMode %d\n",
            pPoolcfg->mmzName, pPoolcfg->blkSize,
            pPoolcfg->blkCnt, pPoolcfg->enRemapMode);
    }
}

ES_VOID dmabuf_statistic_check(ES_VOID)
{
    system("cat /sys/kernel/debug/dma_buf/bufinfo");
}

ES_S32 default_clean_up(void)
{
    ES_VB_MemDebugInfo();
    dmabuf_statistic_check();

    return 0;
}

int dma_memcp_f2f(ES_U64 src_fd, ES_U32 src_offset,
                  ES_U64 dst_fd, ES_U32 dst_offset, ES_U64 len, ES_S32 timeout)
{
    // struct timeval tBegin, tEnd;
    ES_S32 cmdQHandle;
    ES_U32 ret = 0;

    ret = ES_CmdQueue_Create(&cmdQHandle);
    if(ret) {
        ES_MEMCP_ERR("ES_CmdQ_Create fail\n");
        return -1;
    }

    ret = ES_Memcpy_Async(cmdQHandle, src_fd, src_offset, dst_fd, dst_offset, len, timeout);
    if (ret) {
        perror("ES_Memcpy_Async cmdQHandle failed");
        return -1;
    }
    ES_MEMCP_INFO("Task added to cmdQHandle\n");

    struct esw_cmdq_query q1;

    ret = ES_CmdQueue_Query(cmdQHandle, &q1);
    if (ret) {
        perror("ES_CmdQueue_Query cmdQHandle failed");
        return -1;
    }
    ES_MEMCP_INFO("[INFO] CmdQueue_Query (After Sync): status=%d, task_count=%d",
       q1.status, q1.task_count);

    ret = ES_CmdQueue_Sync(cmdQHandle);
    if (ret) {
        perror("ES_CmdQueue_Sync failed");
        return -1;
    }

    ret = ES_CmdQueue_Query(cmdQHandle, &q1);
    if (ret) {
        perror("ES_CmdQueue_Query cmdQHandle failed");
        return -1;
    }
    ES_MEMCP_INFO("[INFO] CmdQueue_Query (After Sync): status=%d, task_count=%d\n",
       q1.status, q1.task_count);

    ret = ES_CmdQueue_Destroy(cmdQHandle);
    if (ret) {
        perror("ES_CmdQueue_Destroy failed");
        return -1;
    }

    ES_MEMCP_INFO("CmdQueue destroyed!\n");

    return ret;
}

ES_S32 system_mem_test(void)
{
    ES_S32 ret = 0;
    ES_U64 fd_src = 0;
    ES_U64 fd_dst = 0;
    ES_VOID *pVirAddr_src = NULL;
    ES_VOID *pVirAddr_dst = NULL;
    ES_S32 test_result = 0, test_result_internal = 0;
    struct cache_info *src = NULL;
    struct cache_info *dst = NULL;
    ES_U64 mem_len = 0;
    ES_S32 loop_cnt = 0;
    ES_S32 item_num = 0;
    ES_S32 timeout = 5000;

    for (loop_cnt = 0; loop_cnt < sizeof(sys_test_len) / sizeof(ES_U64); loop_cnt++) {
        mem_len = sys_test_len[loop_cnt];
        ES_MEMCP_INFO("!!!!!!!!!!!!!!!!!!!mem_len:0x%llx!!!!!!!!!!!!!!!!!!!!\n", mem_len);

        for (item_num = 0; item_num < sizeof(sys_test_data) / sizeof(struct sys_testdata_info); item_num++) {
            ret = 0;
            fd_src = 0;
            fd_dst = 0;
            pVirAddr_src = NULL;
            pVirAddr_dst = NULL;
            src = NULL;
            dst = NULL;
            test_result_internal = 0;

            ES_MEMCP_INFO("=====================sys mem test mem_len:0x%llx item_num:%d===============\n", mem_len, item_num);
            src = &sys_test_data[item_num].src;
            dst = &sys_test_data[item_num].dst;

            // Allocate source memory
            ret = ES_SYS_MemAlloc(&fd_src, src->kernel_cache_type, "src", src->zone, mem_len);
            if (ES_SUCCESS != ret || fd_src <= 0) {
                ES_MEMCP_ERR("ES_SYS_MemAlloc failed for src, ret %x, fd_src: %lld\n", ret, fd_src);
                test_result_internal++;
                goto release;
            }

            // Allocate destination memory
            ret = ES_SYS_MemAlloc(&fd_dst, dst->kernel_cache_type, "dst", dst->zone, mem_len);
            if (ES_SUCCESS != ret || fd_dst <= 0) {
                ES_MEMCP_ERR("ES_SYS_MemAlloc failed for dst, ret %x, fd_dst: %lld\n", ret, fd_dst);
                test_result_internal++;
                goto release;
            }

            // Map source and destination addresses
            pVirAddr_src = ES_SYS_Mmap(fd_src, mem_len, src->mmap_cache_type);
            if (!pVirAddr_src) {
                ES_MEMCP_ERR("ES_SYS_Mmap failed for src!\n");
                test_result_internal++;
                goto release;
            }

            pVirAddr_dst = ES_SYS_Mmap(fd_dst, mem_len, dst->mmap_cache_type);
            if (!pVirAddr_dst) {
                ES_MEMCP_ERR("ES_SYS_Mmap failed for dst!\n");
                test_result_internal++;
                goto release;
            }

            // Fill source buffer
            buf_fillin_random(pVirAddr_src, mem_len, PRIME_SEED_A);
            if (src->mmap_cache_type == SYS_CACHE_MODE_CACHED) {
                ret = ES_SYS_MemFlushCache(fd_src);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_MemFlushCache failed for src, ret %x!\n", ret);
                    test_result_internal++;
                    goto release;
                }
            }

            // Memory copy is performed using DMA
            if (fd_src > 0 && fd_dst > 0) {
                ret = dma_memcp_f2f(fd_src, 0, fd_dst, 0, mem_len, timeout);
                if (ret < 0) {
                    ES_MEMCP_ERR("DMA memcopy failed, ret %d, errno: %d\n", ret, errno);
                    test_result_internal++;
                    goto release;
                }
            } else {
                ES_MEMCP_ERR("Invalid file descriptor for DMA memcopy\n");
                test_result_internal++;
                goto release;
            }

            // Verify the contents of the target buffer
            if (dst->mmap_cache_type == SYS_CACHE_MODE_CACHED) {
                ret = ES_SYS_MemFlushCache(fd_dst);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_MemFlushCache failed for dst, ret %x!\n", ret);
                    test_result_internal++;
                    goto release;
                }
            }

            // Compare source and target data
            if (buf_compwith_random(pVirAddr_dst, mem_len, PRIME_SEED_A)) {
                ES_MEMCP_INFO("DMA test failed for mem_len: 0x%llx, item_num: %d\n", mem_len, item_num);
                test_result_internal++;
            } else {
                ES_MEMCP_INFO("DMA test successfully for mem_len: 0x%llx, item_num: %d\n", mem_len, item_num);
            }

        release:
            // Clear mappings and memory
            if (pVirAddr_src) {
                ret = ES_SYS_Munmap(pVirAddr_src, mem_len);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_Munmap failed for src, ret %x!\n", ret);
                    test_result_internal++;
                }
            }

            if (pVirAddr_dst) {
                ret = ES_SYS_Munmap(pVirAddr_dst, mem_len);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_Munmap failed for dst, ret %x!\n", ret);
                    test_result_internal++;
                }
            }

            if (fd_src) {
                ret = ES_SYS_MemFree(fd_src);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_MemFree failed for src, ret %x!\n", ret);
                    test_result_internal++;
                }
            }

            if (fd_dst) {
                ret = ES_SYS_MemFree(fd_dst);
                if (ES_SUCCESS != ret) {
                    ES_MEMCP_ERR("ES_SYS_MemFree failed for dst, ret %x!\n", ret);
                    test_result++;
                }
            }

            if (test_result_internal != 0) {
                ES_MEMCP_ERR("==================test failed for mem_len: 0x%llx, item_num: %d================\n", mem_len, item_num);
                test_result++;
            } else {
                ES_MEMCP_INFO("==================test success for mem_len: 0x%llx, item_num: %d================\n", mem_len, item_num);
            }
        }
    }

    return test_result;
}

ES_S32 VB_mem_mod_test ()
{
    ES_S32 i = 0, j = 0;
    VB_CONFIG_S vbConfig_in;
    VB_CONFIG_S vbConfig_out;
    VB_POOL_CONFIG_S *poolcfg = NULL;
    VB_UID_E enVbUid = VB_UID_VENC;
    ES_CHAR *mmzName = "mmz_nid_0_part_0";
    ES_S32 ret;
    ES_VOID *pVirAddr_src = NULL;
    ES_VOID *pVirAddr_dst = NULL;
    ES_U64 src_memFd = 0;
    ES_U64 dst_memFd = 0;
    ES_U64 memsize = 4096;
    ES_S32 vb_case_fail_num = 0;
    ES_S32 timeout = 5000;

    for (i = 0; i < sizeof(vb_normal_cases) / sizeof(struct case_vb_config_param_t);i++) {
        if(memsize > vb_normal_cases[i].blkSize)
            continue;

        memset(&vbConfig_in,0,sizeof(VB_CONFIG_S));
        memset(&vbConfig_out,0,sizeof(VB_CONFIG_S));

        vbConfig_in.poolCnt = vb_normal_cases[i].poolCnt;

        for (j = 0;j < vbConfig_in.poolCnt;j++) {
            poolcfg = &vbConfig_in.poolCfgs[j];
            poolcfg->blkSize = vb_normal_cases[i].blkSize;
            poolcfg->blkCnt = vb_normal_cases[i].blkCnt;
            poolcfg->enRemapMode =  vb_normal_cases[i].enRemapMode;
            memcpy(poolcfg->mmzName, vb_normal_cases[i].mmzName, strlen(mmzName));
        }

        ES_VB_PrintConfig(&vbConfig_in);
        ret = ES_VB_SetModPoolConfig(enVbUid, &vbConfig_in);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("SetModConfig, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        }

        ret = ES_VB_GetModPoolConfig(enVbUid,&vbConfig_out);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_VB_GetModPoolConfig, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        }

        for (j = 0;j < vbConfig_in.poolCnt;j++) {
            if ((vbConfig_in.poolCfgs[j].blkCnt != vbConfig_out.poolCfgs[j].blkCnt) ||
                (vbConfig_in.poolCfgs[j].blkSize != vbConfig_out.poolCfgs[j].blkSize) ||
                (vbConfig_in.poolCfgs[j].enRemapMode != vbConfig_out.poolCfgs[j].enRemapMode)) {
                ES_MEMCP_ERR("error:compare poolcfg failed!\n");
                vb_case_fail_num++;
            goto release_mem;
            }

            if (strlen(vbConfig_in.poolCfgs[j].mmzName)) {
                if (memcmp(vbConfig_in.poolCfgs[j].mmzName,
                    vbConfig_out.poolCfgs[j].mmzName,ES_MAX_MMZ_NAME_LEN) != 0) {
                    ES_MEMCP_ERR("error:compare poolcfg mmzName faild!\n");
                    vb_case_fail_num++;
                    goto release_mem;
                }
            } else {
                if (memcmp(vbConfig_out.poolCfgs[j].mmzName,
                    DEFAULT_NID0_MMZ_NAME, strlen(DEFAULT_NID0_MMZ_NAME)) != 0) {
                    ES_MEMCP_ERR("error:compare poolcfg mmzName(%s) faild!\n",
                        vbConfig_out.poolCfgs[j].mmzName);
                    vb_case_fail_num++;
                    goto release_mem;
                }
            }
        }

        ret = ES_VB_ModPoolInit(enVbUid);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("error:ES_VB_ModPoolInit, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        }

        ret = ES_VB_GetModBlock(enVbUid,memsize, vb_normal_cases[i].mmzName, &src_memFd);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_VB_GetModBlock, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        } else {
            ES_MEMCP_INFO("ES_VB_GetModBlock src_fd %llx\n", src_memFd);
        }
        ret = ES_VB_GetModBlock(enVbUid,memsize, vb_normal_cases[i].mmzName, &dst_memFd);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_VB_GetModBlock, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        } else {
            ES_MEMCP_INFO("ES_VB_GetModBlock dst_fd %llx\n", src_memFd);
        }

        pVirAddr_src = ES_SYS_Mmap(src_memFd, memsize, SYS_CACHE_MODE_CACHED);
        if (!pVirAddr_src) {
                ES_MEMCP_ERR("ES_SYS_Mmap failed for src!\n");
                vb_case_fail_num++;
                goto release_mem;
        }
        ES_MEMCP_INFO("ES_SYS_Mmap memFd:%llx\n", src_memFd);

        sprintf(pVirAddr_src, "hello world!!");
        ES_MEMCP_INFO("memset pVirAddr:%p\n", pVirAddr_src);

        ret = ES_SYS_MemFlushCache(src_memFd);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_SYS_MemFlushCache failed, ret %x!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        }

        if (src_memFd > 0 && dst_memFd > 0) {
            ret = dma_memcp_f2f(src_memFd, 0, dst_memFd, 0, memsize, timeout);
            if (ret < 0) {
                ES_MEMCP_ERR("DMA memcopy failed, ret %d, errno: %d\n", ret, errno);
                vb_case_fail_num++;
                goto release_mem;
            }
        } else {
            ES_MEMCP_ERR("Invalid file descriptor for DMA memcopy\n");
            vb_case_fail_num++;
            goto release_mem;
        }

        pVirAddr_dst = ES_SYS_Mmap(dst_memFd, memsize, SYS_CACHE_MODE_CACHED);
        if (!pVirAddr_dst) {
                ES_MEMCP_ERR("ES_SYS_Mmap failed for dst!\n");
                vb_case_fail_num++;
                goto release_mem;
        }
        ES_MEMCP_INFO("ES_SYS_Mmap memFd:%llx\n", dst_memFd);

        ret = ES_SYS_MemFlushCache(dst_memFd);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_SYS_MemFlushCache failed, ret %d!\n", ret);
            vb_case_fail_num++;
            goto release_mem;
        }

        printf("read from dmabuf mmap: %s\n", (char*)pVirAddr_dst);

        if (memcmp((char*)pVirAddr_src,    (char*)pVirAddr_dst, strlen(pVirAddr_src)) != 0) {
            ES_MEMCP_INFO("DMA test failed for mem_len: 0x%llx, item_num: %d\n", memsize, i);
            vb_case_fail_num++;
            goto release_mem;
        } else {
            ES_MEMCP_INFO("DMA test successfully for mem_len: 0x%llx, item_num: %d\n", memsize, i);
        }

        dmabuf_statistic_check();

release_mem:

        if (pVirAddr_src) {
            ret = ES_SYS_Munmap(pVirAddr_src, memsize);
            if (ES_SUCCESS != ret) {
                ES_MEMCP_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
                vb_case_fail_num++;
            }
        }

        if (pVirAddr_dst) {
            ret = ES_SYS_Munmap(pVirAddr_dst, memsize);
            if (ES_SUCCESS != ret) {
                ES_MEMCP_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
                vb_case_fail_num++;
            }
        }

        if (src_memFd) {
            ret = ES_VB_ReleaseBlock (src_memFd);
            if (ES_SUCCESS != ret) {
                ES_MEMCP_ERR("ES_VB_ReleaseBlock, ret %x!\n", ret);
                vb_case_fail_num++;
            } else {
                ES_MEMCP_INFO("ES_VB_ReleaseBlock src_fd %llx\n", src_memFd);
            }
        }

        if (dst_memFd) {
            ret = ES_VB_ReleaseBlock (dst_memFd);
            if (ES_SUCCESS != ret) {
                ES_MEMCP_ERR("ES_VB_ReleaseBlock, ret %x!\n", ret);
                vb_case_fail_num++;
            } else {
                ES_MEMCP_INFO("ES_VB_ReleaseBlock src_fd %llx\n", dst_memFd);
            }
        }

        /*destory pool*/
        ret = ES_VB_ModPoolExit(enVbUid);
        if (ES_SUCCESS != ret) {
            ES_MEMCP_ERR("ES_VB_ModPoolExit, ret %x!\n", ret);
            vb_case_fail_num++;
        } else {
            ES_MEMCP_INFO("ES_VB_ModPoolExit SUCCESS!\n");
        }
    }

    ES_U32 num = sizeof(vb_normal_cases) / sizeof(struct case_vb_config_param_t);
    ES_MEMCP_INFO("==================VB: %d passed, %d failed, 0 error==================\n",
        num - vb_case_fail_num, vb_case_fail_num);

    return vb_case_fail_num;
}



static struct test_case_t dma_mem_test_cases[] = {
    {
        .name = "1.system memory test",
        .test_fn = system_mem_test,
        .cleanup = default_clean_up,
    },
    {
        .name = "2.VB mod memory normal test",
        .test_fn = VB_mem_mod_test,
        .cleanup = default_clean_up,
    }
};

int main(int argc, char* argv[]) {
    ES_S32 ret = 0;
    ES_S32 case_num = 0;
    ES_S32 arg = -1;
    ES_S32 exit_state = 0;
    ES_S32 round_cnt = 0;
    ES_S32 case_fail_num = 0;
    ES_S32 case_total_num = sizeof(dma_mem_test_cases) / sizeof(struct test_case_t);

    printf("ES_MAX_MMZ_NAME_LEN: %d\n", ES_MAX_MMZ_NAME_LEN);

    if (access("/sys/kernel/debug/dma_buf", F_OK) == -1) {
        printf("mount -t debugfs none /sys/kernel/debug\n");
        system("mount -t debugfs none /sys/kernel/debug");
    }

    printf("case_total_num=%d\n", case_total_num);

    // Gets the run use case parameters
    if (argc > 1) {
        arg = atoi(argv[1]);
        if (arg > case_total_num || arg < 1) {
            printf("error: wrong input (%d)\n", arg);
            return -1;
        }
        case_num = arg - 1;
    } else {
        case_num = -1; // -1 Indicates run all use cases
    }

    // Run the specified test case
    if (case_num != -1) {
        printf("-----------------case(%s)---------------\n", dma_mem_test_cases[case_num].name);

        // Preparatory phase
        if (dma_mem_test_cases[case_num].prepare != NULL) {
            ret = dma_mem_test_cases[case_num].prepare();
            if (ret) {
                printf("-----------case prepare failed!--------\n");
                return -1;
            }
        }

        // Test phase
        if (dma_mem_test_cases[case_num].test_fn != NULL) {
            ret = dma_mem_test_cases[case_num].test_fn();
            if (ret) {
                printf("-----------case(%s) test failed!--------\n", dma_mem_test_cases[case_num].name);
                exit_state = -1;
            } else {
                printf("-----------case(%s) test success!--------\n", dma_mem_test_cases[case_num].name);
            }
        }

        // Cleanup phase
        if (dma_mem_test_cases[case_num].cleanup != NULL) {
            dma_mem_test_cases[case_num].cleanup();
        }

        printf("Closed DMA_MEMCP_DRIVER\n");
        return exit_state;
    }

    // Run the logic for all use cases
again:
    case_fail_num = 0;
    for (case_num = 0; case_num < case_total_num; case_num++) {
        printf("-----------------case(%s)---------------\n", dma_mem_test_cases[case_num].name);

        // Preparatory phase
        if (dma_mem_test_cases[case_num].prepare != NULL) {
            ret = dma_mem_test_cases[case_num].prepare();
            if (ret) {
                printf("-----------case prepare failed!--------\n");
                continue;
            }
        }

        // Test phase
        if (dma_mem_test_cases[case_num].test_fn != NULL) {
            ret = dma_mem_test_cases[case_num].test_fn();
            if (ret) {
                printf("-----------case(%s) test failed!--------\n", dma_mem_test_cases[case_num].name);
                dma_mem_test_cases[case_num].testResult = 0;
                case_fail_num++;
            } else {
                printf("-----------case(%s) test success!--------\n", dma_mem_test_cases[case_num].name);
                dma_mem_test_cases[case_num].testResult = 1;
            }
        }

        // Cleanup phase
        if (dma_mem_test_cases[case_num].cleanup != NULL) {
            dma_mem_test_cases[case_num].cleanup();
        }
    }

    // Check test results
    for (case_num = 0; case_num < case_total_num; case_num++) {
        if (dma_mem_test_cases[case_num].testResult != 1) {
            printf("-----------case(%s) test failed!\n", dma_mem_test_cases[case_num].name);
            exit_state = -1;
        }
    }

    if (exit_state == 0) {
        round_cnt++;
        printf("Round count [%d] completed successfully!\n", round_cnt);
        system("date");

        // Repeated testing
        if (arg == 0) goto again;
    }

    printf("Closed DMA_MEMCP_DRIVER\n");
    printf("Exit state: %d\n", exit_state);

    if (argc == 1) {
        printf("*****%d passed, %d failed, 0 error*****\n", case_total_num - case_fail_num, case_fail_num);
    }

    return exit_state;
}
